@@ -131,7 +131,7 @@ class OutOfGraphReplayBuffer(object):
     tf.logging.info(
         'Creating a %s replay memory with the following parameters:',
         self.__class__.__name__)
-    tf.logging.info('\t observation_shape: %d', observation_shape)
+    tf.logging.info('\t observation_shape: %s', str(observation_shape))
     tf.logging.info('\t stack_size: %d', stack_size)
     tf.logging.info('\t replay_capacity: %d', replay_capacity)
     tf.logging.info('\t batch_size: %d', batch_size)
